package jumpstart.web.pages.examples.tables;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;

import org.apache.tapestry5.ValueEncoder;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;

public class EditableGrid1 {
	private final int MAX_RESULTS = 30;

	// Screen fields

	@Property
	private List<Person> persons;

	@Property
	@SuppressWarnings("unused")
	private Person person;

	@Property
	private PersonEncoder personEncoder;

	// Work fields

	private List<Person> personsEdited;

	private boolean oneOrMorePersonsGone;

	// Other pages

	@InjectPage
	private EditableGrid2 page2;

	// Generally useful bits and pieces

	@Component(id = "personsedit")
	private Form form;

	@EJB
	private IPersonServiceLocal personService;

	// The code

	// Form bubbles up the PREPARE_FOR_SUBMIT event during form submission.

	void onPrepareForSubmit() {
		personsEdited = new ArrayList<Person>();
		oneOrMorePersonsGone = false;
	}

	// Form bubbles up the PREPARE event during form render and form submission.

	void onPrepare() {
		if (personEncoder == null) {
			personEncoder = new PersonEncoder();
			// Get all persons - ask business service to find them (from the database)
			persons = personService.findPersons(MAX_RESULTS);
		}
	}

	// This encoder is used by our Grid:
	// - during render, to convert each person to an id (Grid then stores the ids in the form, hidden).
	// - during form submission, to convert each id back to a person which it puts in our person field. We also add it
	// to our list of personsEdited. Grid will overwrite the firstName of the person returned.

	private class PersonEncoder implements ValueEncoder<Person> {

		public String toClient(Person value) {
			return value.getId().toString();
		}

		public Person toValue(String idAsString) {
			Long id = new Long(idAsString);
			Person person = findPerson(id);
			if (person == null) {
				oneOrMorePersonsGone = true;
			}
			else {
				personsEdited.add(person);
			}
			// Grid will overwrite the firstName of the person returned.
			// Grid can't handle null person, so if null we return a new empty person.
			return person == null ? new Person() : person;
		}

		private Person findPerson(Long id) {

			// We could find the person in the database, but it's cheaper to search the list we got in onPrepare().

			for (Person person : persons) {
				if (person.getId().equals(id)) {
					return person;
				}
			}
			return null;
		}
	};

	void onValidateFromPersonsEdit() {

		if (form.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		// Let's say a requirement is to not proceed if a person we displayed is no longer in the database - check it.

		if (oneOrMorePersonsGone) {
			form.recordError("The list of persons is out of date. Please refresh and try again.");
			return;
		}

		try {
			System.out.println(">>> personsEdited = " + personsEdited);
			// In a real application you would persist them to the database instead of printing them
			// personService.changePersons(persons);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			form.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	Object onSuccess() {
		page2.set(personsEdited);
		return page2;
	}
}
