package jumpstart.web.pages.previews.withlayout.gracefulajaxfiltercrud;

import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.List;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.Regions;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;
import jumpstart.web.components.CustomForm;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.PersistenceConstants;
import org.apache.tapestry5.ajax.MultiZoneUpdate;
import org.apache.tapestry5.annotations.ActivationRequestParameter;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectComponent;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.corelib.components.Grid;
import org.apache.tapestry5.corelib.components.Zone;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;

public class Persons {

	private final String demoModeStr = System.getProperty("jumpstart.demo-mode");
	private final int MAX_RESULTS = 40;

	private enum Mode {
		CREATE, REVIEW, UPDATE, CONFIRM_DELETE;
	}

	// The activation context

	@Property
	private Mode mode;

	@Property
	private Long personId;
	
	// Query string parameters

	@Property
	@ActivationRequestParameter(value = "filter")
	private String partialName;

	// Screen fields

	@Property
	@Persist
	private boolean highlightZoneUpdates;

	@Property
	@Persist
	@SuppressWarnings("unused")
	private List<Person> listPersons;

	@Property
	private Person listPerson;

	@Property
	private Long highlightPersonId;

	@Property
	private Person person;

	@Property
	@Persist(PersistenceConstants.FLASH)
	@SuppressWarnings("unused")
	private String deleteMessage;

	@Component(id = "list")
	@Property
	@SuppressWarnings("unused")
	private Grid list;

	// Generally useful bits and pieces

	@EJB
	private IPersonServiceLocal personService;

	@Component(id = "createForm")
	private CustomForm createForm;

	@Component(id = "updateForm")
	private CustomForm updateForm;

	@Component(id = "confirmDeleteForm")
	private Form confirmDeleteForm;

	@InjectComponent
	private Zone listZone;

	@InjectComponent
	private Zone editorZone;

	@Inject
	private Messages messages;

	@Inject
	private Request request;

	// The code

	// onPassivate() is called by Tapestry to get the activation context to put in the URL.

	Object[] onPassivate() {

		if (mode == null) {
			return null;
		}
		else if (mode == Mode.CREATE) {
			return new Object[] { mode };
		}
		else if (mode == Mode.REVIEW || mode == Mode.UPDATE || mode == Mode.CONFIRM_DELETE) {
			return new Object[] { mode, personId };
		}
		else {
			throw new IllegalStateException(mode.toString());
		}

	}

	// onActivate() is called by Tapestry to pass in the activation context from the URL.

	void onActivate(EventContext ec) {

		if (ec.getCount() == 0) {
			mode = null;
			personId = null;
		}
		else if (ec.getCount() == 1) {
			mode = ec.get(Mode.class, 0);
			personId = null;
		}
		else {
			mode = ec.get(Mode.class, 0);
			personId = ec.get(Long.class, 1);
		}

	}

	// setupRender() is called by Tapestry right before it starts rendering the page.

	void setupRender() {
		setupList();
		setupEditor();
	}

	void setupList() {
		listPersons = personService.findPersons(partialName, MAX_RESULTS);
		highlightPersonId = personId;
		if (highlightPersonId == null) {
			// Null is a nuisance because it fails to match the signature of our onSuccess method.
			highlightPersonId = -1L;
		}
	}

	void setupEditor() {
		if (personId == null) {
			person = null;
		}
		else {
			if (person == null) {
				person = personService.findPerson(personId);
				// Handle null person in the template.
			}
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// FILTER
	// /////////////////////////////////////////////////////////////////////

	Object onSuccessFromFilterForm(Long highlightPersonId) {
		personId = highlightPersonId;
		setupList();
		return request.isXHR() ? listZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toCreate"

	Object onToCreate() {
		mode = Mode.CREATE;
		personId = null;
		setupList();
		setupEditor();
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// Handle event "cancelCreate"

	Object onCancelCreate() {
		mode = null;
		personId = null;
		setupEditor();
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Component "createForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromCreateForm() throws Exception {
		mode = Mode.CREATE;
		// Instantiate a Person for the form data to overlay.
		person = new Person();
	}

	// Component "createForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromCreateForm() {

		if (createForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		if (demoModeStr != null && demoModeStr.equals("true")) {
			createForm.recordError("Sorry, but Create is not allowed in Demo mode.");
			return;
		}

		try {
			person = personService.createPerson(person);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			createForm.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	// Component "createForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether VALIDATE
	// records an error

	Object onSuccessFromCreateForm() {
		mode = Mode.REVIEW;
		personId = person.getId();
		setupList();
		setupEditor();
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	Object onFailureFromCreateForm() {
		setupEditor();
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// Handle event "selected"

	Object onSelected(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		setupList();
		setupEditor();
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate"

	Object onToUpdate(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		setupEditor();
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Handle event "cancelUpdate"

	Object onCancelUpdate(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		setupEditor();
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// Component "updateForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromUpdateForm(Long personId) {
		this.personId = personId;
		if (person == null) {
			// Get objects for the form fields to overlay.
			person = personService.findPerson(personId);
			// Handle null person in the template.
		}
		mode = Mode.UPDATE;
	}

	// Component "updateForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromUpdateForm() {

		if (updateForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		try {
			personService.changePerson(person);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			updateForm.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	// Component "updateForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether VALIDATE
	// records an error

	Object onSuccessFromUpdateForm() {
		mode = Mode.REVIEW;
		personId = person.getId();
		setupList();
		setupEditor();
		return request.isXHR() ? getListAndEditorZones() : null;
	}

	Object onFailedUpdate(Long personId) {
		setupEditor();
		return request.isXHR() ? editorZone.getBody() : null;
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "delete"

	Object onDelete(Long personId, Integer personVersion) {

		// If request is AJAX then the user has pressed Delete..., was presented with a Confirm dialog, and OK'd it.

		if (request.isXHR()) {
			mode = Mode.REVIEW;
			this.personId = personId;
			person = personService.findPerson(personId);
			// Handle null person in the template.

			boolean successfulDelete = false;

			if (person != null) {

				if (demoModeStr != null && demoModeStr.equals("true")) {
					deleteMessage = "Sorry, but Delete is not allowed in Demo mode.";
				}
				else {

					// Delete the person from the database unless they've been modified elsewhere

					try {
						if (!person.getVersion().equals(personVersion)) {
							deleteMessage = "Cannot delete person because has been updated or deleted since last displayed. Please refresh and try again.";
						}
						else {
							personService.deletePerson(person);
							successfulDelete = true;
						}
					}
					catch (Exception e) {
						// Display the cause. In a real system we would try harder to get a user-friendly message.
						deleteMessage = ExceptionUtil.getRootCauseMessage(e);
					}
				}
			}

			if (successfulDelete) {
				mode = null;
				this.personId = null;
				setupList();
				setupEditor();
				return getListAndEditorZones();
			}
			else {
				mode = Mode.REVIEW;
				this.personId = personId;
				setupEditor();
				return editorZone.getBody();
			}

		}

		// Else, (JavaScript disabled) user has pressed Delete..., but not yet confirmed so go to a confirmation screen.

		else {
			mode = Mode.CONFIRM_DELETE;
			this.personId = personId;
			person = personService.findPerson(personId);
			// Handle null person in the template.

			return null;
		}

	}

	// /////////////////////////////////////////////////////////////////////
	// CONFIRM DELETE - used only when JavaScript is disabled.
	// /////////////////////////////////////////////////////////////////////

	// Handle event "cancelConfirmDelete"

	Object onCancelConfirmDelete(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		return null;
	}

	// Component "confirmDeleteForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromConfirmDeleteForm() {
		mode = Mode.CONFIRM_DELETE;
		if (person == null) {
			// Get objects for the form fields to overlay.
			person = personService.findPerson(personId);
			// Handle null person in the template.
		}
	}

	// Component "confirmDeleteForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromConfirmDeleteForm(Long personId, Integer personVersion) {
		this.personId = personId;

		if (confirmDeleteForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		if (person != null) {

			if (demoModeStr != null && demoModeStr.equals("true")) {
				confirmDeleteForm.recordError("Sorry, but Delete is not allowed in Demo mode.");
			}
			else {

				// Delete the person from the database unless they've been modified elsewhere

				try {
					if (!person.getVersion().equals(personVersion)) {
						confirmDeleteForm
								.recordError("Cannot delete person because has been updated or deleted since last displayed. Please refresh and try again.");
					}
					else {
						personService.deletePerson(person);
					}
				}
				catch (Exception e) {
					// Display the cause. In a real system we would try harder to get a user-friendly message.
					confirmDeleteForm.recordError(ExceptionUtil.getRootCauseMessage(e));
				}
			}
		}

	}

	// Component "confirmDeleteForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether
	// VALIDATE records an error

	void onSuccessFromConfirmDeleteForm() {
		mode = null;
		personId = null;
		setupList();
		setupEditor();
	}

	// /////////////////////////////////////////////////////////////////////
	// OTHER
	// /////////////////////////////////////////////////////////////////////

	public String getLinkCSSClass() {
		if (listPerson != null && listPerson.getId().equals(highlightPersonId)) {
			return "active";
		}
		else {
			return "";
		}
	}

	private MultiZoneUpdate getListAndEditorZones() {
		return new MultiZoneUpdate("listZone", listZone.getBody()).add("editorZone", editorZone.getBody());
	}

	public String getZoneUpdateFunction() {
		return highlightZoneUpdates ? "highlight" : "show";
	}

	public boolean isModeCreate() {
		return mode == Mode.CREATE;
	}

	public boolean isModeReview() {
		return mode == Mode.REVIEW;
	}

	public boolean isModeUpdate() {
		return mode == Mode.UPDATE;
	}

	public boolean isModeConfirmDelete() {
		return mode == Mode.CONFIRM_DELETE;
	}

	public boolean isModeNull() {
		return mode == null;
	}

	public boolean isConfirmDeleteFormValid() {
		return confirmDeleteForm.isValid();
	}

	public String getPersonRegion() {
		// Follow the same naming convention that the Select component uses
		return messages.get(Regions.class.getSimpleName() + "." + person.getRegion().name());
	}

	public String getDatePattern() {
		return "dd/MM/yyyy";
	}

	public Format getDateFormat() {
		return new SimpleDateFormat(getDatePattern());
	}
}
