package jumpstart.web.pages.theapp.security;

import javax.ejb.EJB;

import jumpstart.business.commons.exception.DoesNotExistException;
import jumpstart.business.domain.security.User;
import jumpstart.business.domain.security.User.PageStyle;
import jumpstart.business.domain.security.iface.ISecurityFinderServiceLocal;
import jumpstart.business.domain.security.iface.ISecurityManagerServiceLocal;
import jumpstart.web.annotation.ProtectedPage;
import jumpstart.web.base.theapp.SimpleBasePage;

import org.apache.tapestry5.PersistenceConstants;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.util.EnumValueEncoder;

@ProtectedPage
public class MyPreferencesEdit extends SimpleBasePage {

	// Screen fields

	@Property
	private User user;
	
	@Property
	@Persist(PersistenceConstants.FLASH)
	@SuppressWarnings("unused")
	private String message; 

	@Property
	@SuppressWarnings({ "rawtypes", "unchecked", "unused" })
	private EnumValueEncoder pageStyleEncoder = new EnumValueEncoder(User.PageStyle.class);

	// Generally useful bits and pieces

	@Component(id = "form")
	private Form form;
	
	@EJB
	private ISecurityFinderServiceLocal securityFinderService;
	
	@EJB
	private ISecurityManagerServiceLocal securityManagerService;

	// The code
	
	void onPrepare() throws DoesNotExistException {
		Long userId = getVisit().getMyUserId();
		user = securityFinderService.findUser(userId);
	}
	
	void onValidateFromForm() {
		try {
			securityManagerService.changeUser(user);
		}
		catch (Exception e) {
			form.recordError(interpretBusinessServicesExceptionForChange(e));
		}
	}

	void onSuccess() {
		getVisit().noteChanges(user);
		message = getMessages().get("User_preferences_saved");
	}

	void onRefresh() {
	}

	public PageStyle getBoxy() {
		return User.PageStyle.BOXY;
	}

	public PageStyle getWide() {
		return User.PageStyle.WIDE;
	}

	public String[] getDatePatterns() {
		return User.DATE_PATTERNS;
	}
}
