package jumpstart.web.pages.theapp.security;

import javax.ejb.EJB;

import jumpstart.business.commons.exception.DoesNotExistException;
import jumpstart.business.domain.security.UserRole;
import jumpstart.business.domain.security.iface.ISecurityFinderServiceLocal;
import jumpstart.business.domain.security.iface.ISecurityManagerServiceLocal;
import jumpstart.web.annotation.ProtectedPage;
import jumpstart.web.base.theapp.SimpleBasePage;

import org.apache.tapestry5.Link;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;

@ProtectedPage
public class UserRoleEdit extends SimpleBasePage {

	// Screen fields
	
	@Property
	private Long userRoleId;
	
	@Persist
	private Link returnTo;

	@Property
	private UserRole userRole;

	// Generally useful bits and pieces
	
	@Component(id = "form")
	private Form form;
	
	@EJB
	private ISecurityFinderServiceLocal securityFinderService;
	
	@EJB
	private ISecurityManagerServiceLocal securityManagerService;

	// The code
	
	public void set(Long userRoleId, Link returnTo) {
		this.userRoleId = userRoleId;
		this.returnTo = returnTo;
	}
	
	Long onPassivate() {
		return userRoleId;
	}

	void onActivate(Long userRoleId) {
		this.userRoleId = userRoleId;
	}

	void onPrepare() throws DoesNotExistException {
		userRole = securityFinderService.findUserRoleShallowish(userRoleId);
	}
	
	void onValidateFromForm() {
		try {
			securityManagerService.changeUserRole(userRole);
		}
		catch (Exception e) {
			form.recordError(interpretBusinessServicesExceptionForChange(e));
		}
	}

	Link onSuccess() {
		return returnTo;
	}

	void onRefresh() {
	}

	Link onCancel() {
		return returnTo;
	}
}
