package jumpstart.web.services;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLConnection;

import org.apache.tapestry5.ioc.services.ClasspathURLConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ClasspathURLConverterJBoss60 implements ClasspathURLConverter {
	private static Logger logger = LoggerFactory.getLogger(ClasspathURLConverterJBoss60.class);

	public URL convert(URL url) {

		// If the URL is a virtual URL (JBoss 6.0 uses a Virtual File System)...
		
		if (url != null && url.getProtocol().startsWith("vfs")) {

			// Ask the VFS what the physical URL is...
			
			try {
				URLConnection connection = url.openConnection();
				Object jBossVirtualFile = connection.getContent();
				// Use reflection so that we don't need JBoss in the classpath at compile time.
				File physicalFile = (File) invoke(jBossVirtualFile, "getPhysicalFile");
				URL physicalFileURL = physicalFile.toURI().toURL();
				return physicalFileURL;
			}
			catch (Exception e) {
				logger.error(e.getCause().toString());
			}
		}
		
		return url;
	}

	private Object invoke(Object target, String methodName) throws NoSuchMethodException, InvocationTargetException,
			IllegalAccessException {
		Class<?> type = target.getClass();
		Method method;
		try {
			method = type.getMethod(methodName);
		}
		catch (NoSuchMethodException e) {
			method = type.getDeclaredMethod(methodName);
			method.setAccessible(true);
		}
		return method.invoke(target);
	}

}
